@@ -27,6 +27,7 @@
 #include "constraints.h"
 #include "workspace.h"
 #include "place.h"
+#include "prefs.h"
 
 #include <stdlib.h>
 #include <math.h>
@@ -91,7 +92,7 @@
 
 typedef enum
 {
-  PRIORITY_MINIMUM = 0, // Dummy value used for loop start = min(all priorities)
+  PRIORITY_MINIMUM = 0, /* Dummy value used for loop start = min(all priorities) */
   PRIORITY_ASPECT_RATIO = 0,
   PRIORITY_ENTIRELY_VISIBLE_ON_SINGLE_XINERAMA = 0,
   PRIORITY_ENTIRELY_VISIBLE_ON_WORKAREA = 1,
@@ -101,7 +102,7 @@ typedef enum
   PRIORITY_SIZE_HINTS_LIMITS = 3,
   PRIORITY_TITLEBAR_VISIBLE = 4,
   PRIORITY_PARTIALLY_VISIBLE_ON_WORKAREA = 4,
-  PRIORITY_MAXIMUM = 4  // Dummy value used for loop end = max(all priorities)
+  PRIORITY_MAXIMUM = 4 /* Dummy value used for loop end = max(all priorities) */
 } ConstraintPriority;
 
 typedef enum
@@ -392,7 +393,27 @@ setup_constraint_info (ConstraintInfo      *info,
   meta_window_get_work_area_for_xinerama (window,
                                           xinerama_info->number,
                                           &info->work_area_xinerama);
-  info->entire_xinerama = xinerama_info->rect;
+
+  if (!window->fullscreen || window->fullscreen_monitors[0] == -1)
+    {
+      info->entire_xinerama = xinerama_info->rect;
+    }
+  else
+    {
+      int i = 0;
+      long monitor;
+
+      monitor = window->fullscreen_monitors[i];
+      info->entire_xinerama =
+        window->screen->xinerama_infos[monitor].rect;
+      for (i = 1; i <= 3; i++)
+        {
+          monitor = window->fullscreen_monitors[i];
+          meta_rectangle_union (&info->entire_xinerama,
+                                &window->screen->xinerama_infos[monitor].rect,
+                                &info->entire_xinerama);
+        }
+    }
 
   cur_workspace = window->screen->active_workspace;
   info->usable_screen_region   = 
@@ -404,7 +425,8 @@ setup_constraint_info (ConstraintInfo      *info,
   /* Workaround braindead legacy apps that don't know how to
    * fullscreen themselves properly.
    */
-  if (meta_rectangle_equal (new, &xinerama_info->rect) &&
+  if (meta_prefs_get_force_fullscreen() &&
+      meta_rectangle_equal (new, &xinerama_info->rect) &&
       window->has_fullscreen_func &&
       !window->fullscreen)
     {
@@ -784,7 +806,9 @@ constrain_fullscreen (MetaWindow         *window,
   /* Determine whether constraint applies; exit if it doesn't */
   if (!window->fullscreen)
     return TRUE;
+
   xinerama = info->entire_xinerama;
+
   get_size_limits (window, info->fgeom, FALSE, &min_size, &max_size);
   too_big =   !meta_rectangle_could_fit_rect (&xinerama, &min_size);
   too_small = !meta_rectangle_could_fit_rect (&max_size, &xinerama);
@@ -811,6 +835,7 @@ constrain_size_increments (MetaWindow         *window,
   int bh, hi, bw, wi, extra_height, extra_width;
   int new_width, new_height;
   gboolean constraint_already_satisfied;
+  MetaRectangle *start_rect;
 
   if (priority > PRIORITY_SIZE_HINTS_INCREMENTS)
     return TRUE;
@@ -855,7 +880,6 @@ constrain_size_increments (MetaWindow         *window,
   /* Figure out what original rect to pass to meta_rectangle_resize_with_gravity
    * See bug 448183
    */
-  MetaRectangle *start_rect;
   if (info->action_type == ACTION_MOVE_AND_RESIZE)
     start_rect = &info->current;
   else
@@ -879,6 +903,7 @@ constrain_size_limits (MetaWindow         *window,
   MetaRectangle min_size, max_size;
   gboolean too_big, too_small, constraint_already_satisfied;
   int new_width, new_height;
+  MetaRectangle *start_rect;
 
   if (priority > PRIORITY_SIZE_HINTS_LIMITS)
     return TRUE;
@@ -911,7 +936,6 @@ constrain_size_limits (MetaWindow         *window,
   /* Figure out what original rect to pass to meta_rectangle_resize_with_gravity
    * See bug 448183
    */
-  MetaRectangle *start_rect;
   if (info->action_type == ACTION_MOVE_AND_RESIZE)
     start_rect = &info->current;
   else
@@ -936,6 +960,7 @@ constrain_aspect_ratio (MetaWindow         *window,
   int fudge, new_width, new_height;
   double best_width, best_height;
   double alt_width, alt_height;
+  MetaRectangle *start_rect;
 
   if (priority > PRIORITY_ASPECT_RATIO)
     return TRUE;
@@ -1044,7 +1069,6 @@ constrain_aspect_ratio (MetaWindow         *window,
   /* Figure out what original rect to pass to meta_rectangle_resize_with_gravity
    * See bug 448183
    */
-  MetaRectangle *start_rect;
   if (info->action_type == ACTION_MOVE_AND_RESIZE)
     start_rect = &info->current;
   else
@@ -1070,13 +1094,16 @@ do_screen_and_xinerama_relative_constraints (
   MetaRectangle how_far_it_can_be_smushed, min_size, max_size;
 
 #ifdef WITH_VERBOSE_MODE
-  /* First, log some debugging information */
-  char spanning_region[1 + 28 * g_list_length (region_spanning_rectangles)];
+  if (meta_is_verbose ())
+    {
+      /* First, log some debugging information */
+      char spanning_region[1 + 28 * g_list_length (region_spanning_rectangles)];
 
-  meta_topic (META_DEBUG_GEOMETRY,
-              "screen/xinerama constraint; region_spanning_rectangles: %s\n",
-              meta_rectangle_region_to_string (region_spanning_rectangles, ", ", 
-                                               spanning_region));
+      meta_topic (META_DEBUG_GEOMETRY,
+             "screen/xinerama constraint; region_spanning_rectangles: %s\n",
+             meta_rectangle_region_to_string (region_spanning_rectangles, ", ",
+                                              spanning_region));
+    }
 #endif
 
   /* Determine whether constraint applies; exit if it doesn't */
