@@ -54,9 +54,11 @@
 #include "prefs.h"
 
 #include <glib-object.h>
+#include <glib/gprintf.h>
 
 #include <stdlib.h>
 #include <sys/types.h>
+#include <wait.h>
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
@@ -113,11 +115,23 @@ log_handler (const gchar   *log_domain,
 static void
 version (void)
 {
+  const int latest_year = 2009;
+  char yearbuffer[256];
+  GDate date;
+
+  /* this is all so the string to translate stays constant.
+   * see how much we love the translators.
+   */
+  g_date_set_dmy (&date, 1, G_DATE_JANUARY, latest_year);
+  if (g_date_strftime (yearbuffer, sizeof (yearbuffer), "%Y", &date)==0)
+    /* didn't work?  fall back to decimal representation */
+    g_sprintf (yearbuffer, "%d", latest_year);
+
   g_print (_("metacity %s\n"
-             "Copyright (C) 2001-2008 Havoc Pennington, Red Hat, Inc., and others\n"
+             "Copyright (C) 2001-%s Havoc Pennington, Red Hat, Inc., and others\n"
              "This is free software; see the source for copying conditions.\n"
              "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
-           VERSION);
+           VERSION, yearbuffer);
   exit (0);
 }
 
@@ -219,6 +233,7 @@ typedef struct
   gboolean sync;
   gboolean composite;
   gboolean no_composite;
+  gboolean no_force_fullscreen;
 } MetaArguments;
 
 #ifdef HAVE_COMPOSITE_EXTENSIONS
@@ -300,6 +315,12 @@ meta_parse_options (int *argc, char ***argv,
       N_("Turn compositing off"),
       NULL
     },
+    {
+      "no-force-fullscreen", 0, COMPOSITE_OPTS_FLAGS, G_OPTION_ARG_NONE,
+      &my_args.no_force_fullscreen,
+      N_("Don't make fullscreen windows that are maximized and have no decorations"),
+      NULL
+    },
     {NULL}
   };
   GOptionContext *ctx;
@@ -339,10 +360,13 @@ meta_select_display (gchar *display_name)
 static void
 meta_finalize (void)
 {
-  meta_display_close (meta_get_display (),
-                      CurrentTime); /* I doubt correct timestamps matter here */
+  MetaDisplay *display = meta_get_display();
 
   meta_session_shutdown ();
+
+  if (display)
+    meta_display_close (display,
+                        CurrentTime); /* I doubt correct timestamps matter here */
 }
 
 static void
@@ -352,6 +376,29 @@ sigterm_handler (int signum)
 
   exit (meta_exit_code);
 }
+
+static guint sigchld_signal_id = 0;
+
+static void
+sigchld_handler (int signum, siginfo_t *info, void *context)
+{
+  int stat;
+
+  if (info->si_code == CLD_EXITED)
+    {
+      g_signal_emit (sigchld_nexus, sigchld_signal_id, 0,
+                     info->si_status,
+                     GINT_TO_POINTER (info->si_pid));
+    }
+
+  g_signal_handlers_disconnect_matched (sigchld_nexus,
+                                        G_SIGNAL_MATCH_DATA,
+                                        sigchld_signal_id,
+                                        0, NULL, NULL,
+                                        GINT_TO_POINTER (info->si_pid));
+
+  waitpid (info->si_pid, &stat, WNOHANG);
+}
     
 /**
  * This is where the story begins. It parses commandline options and
@@ -375,9 +422,14 @@ main (int argc, char **argv)
     "Pango", "GLib-GObject", "GThread"
   };
   guint i;
+
+  if (!g_thread_supported ())
+    g_thread_init (NULL);
   
   if (setlocale (LC_ALL, "") == NULL)
     meta_warning ("Locale not understood by C library, internationalization will not work\n");
+
+  g_type_init ();
   
   sigemptyset (&empty_mask);
   act.sa_handler = SIG_IGN;
@@ -397,6 +449,24 @@ main (int argc, char **argv)
     g_printerr ("Failed to register SIGTERM handler: %s\n",
 		g_strerror (errno));
 
+  sigchld_nexus = g_object_new (META_TYPE_NEXUS, NULL);
+
+  sigchld_signal_id =
+    g_signal_new ("sigchld", META_TYPE_NEXUS,
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__UINT_POINTER,
+                  G_TYPE_NONE,
+                  2,
+                  G_TYPE_UINT, G_TYPE_POINTER);
+  
+  act.sa_flags = SA_NOCLDSTOP | SA_SIGINFO;
+  act.sa_handler = SIG_DFL;
+  act.sa_sigaction = &sigchld_handler;
+  if (sigaction (SIGCHLD, &act, NULL) < 0)
+    g_printerr ("Failed to register SIGCHLD handler: %s\n",
+		g_strerror (errno));
+
   if (g_getenv ("METACITY_VERBOSE"))
     meta_set_verbose (TRUE);
   if (g_getenv ("METACITY_DEBUG"))
@@ -431,8 +501,6 @@ main (int argc, char **argv)
   
   meta_main_loop = g_main_loop_new (NULL, FALSE);
   
-  g_type_init ();
-
   meta_ui_init (&argc, &argv);  
 
   /* must be after UI init so we can override GDK handlers */
@@ -523,6 +591,9 @@ main (int argc, char **argv)
   if (meta_args.composite || meta_args.no_composite)
     meta_prefs_set_compositing_manager (meta_args.composite);
 
+  if (meta_args.no_force_fullscreen)
+    meta_prefs_set_force_fullscreen (FALSE);
+
   if (!meta_display_open ())
     meta_exit (META_EXIT_ERROR);
   
